@@ -1,16 +1,17 @@
+# https://cloud.google.com/scheduler/docs/reference/rest
 import os
 from typing import Dict
 
 from google.api_core.exceptions import NotFound
 from google.cloud import scheduler
 
-from gcp_pilot.base import GoogleCloudPilotAPI
+from gcp_pilot.base import GoogleCloudPilotAPI, AppEngineBasedService
+
+DEFAULT_TIMEZONE = os.environ.get('TIMEZONE', 'Europe/London')  # UTC
+MAX_TIMEOUT = 30 * 60  # max allowed to HTTP endpoints is 30 minutes
 
 
-DEFAULT_TIMEZONE = os.environ.get('TIMEZONE', 'UTC')
-
-
-class CloudScheduler(GoogleCloudPilotAPI):
+class CloudScheduler(AppEngineBasedService, GoogleCloudPilotAPI):
     _client_class = scheduler.CloudSchedulerClient
     DEFAULT_METHOD = scheduler.HttpMethod.POST
 
@@ -18,11 +19,11 @@ class CloudScheduler(GoogleCloudPilotAPI):
         self.timezone = kwargs.pop('timezone', DEFAULT_TIMEZONE)
         super().__init__(**kwargs)
 
-    def _parent_path(self, project_id=None, location=None):
-        return f'projects/{project_id or self.project_id}/locations/{location or self.location}'
+    def _parent_path(self, project_id: str = None) -> str:
+        return f'projects/{project_id or self.project_id}/locations/{self.location}'
 
-    def _job_path(self, job, project_id=None, location=None):
-        parent_name = self._parent_path(project_id=project_id, location=location)
+    def _job_path(self, job, project_id: str = None) -> str:
+        parent_name = self._parent_path(project_id=project_id)
         return f'{parent_name}/jobs/{job}'
 
     async def create(
@@ -35,19 +36,22 @@ class CloudScheduler(GoogleCloudPilotAPI):
             method: int = DEFAULT_METHOD,
             headers: Dict[str, str] = None,
             project_id: str = None,
-            location: str = None,
-    ):
-        parent = self._parent_path(project_id=project_id, location=location)
-        job_name = self._job_path(job=name, project_id=project_id, location=location)
+            use_oidc_auth: bool = True,
+            timeout_in_seconds: int = MAX_TIMEOUT,
+    ) -> scheduler.Job:
+        parent = self._parent_path(project_id=project_id)
+        job_name = self._job_path(job=name, project_id=project_id)
         job = scheduler.Job(
             name=job_name,
             schedule=cron,
             time_zone=timezone or self.timezone,
+            attempt_deadline=timeout_in_seconds,
             http_target=scheduler.HttpTarget(
                 uri=url,
                 http_method=method,
                 body=payload.encode(),
                 headers=headers or {},
+                **(self.oidc_token if use_oidc_auth else {}),
             )
         )
 
@@ -69,18 +73,21 @@ class CloudScheduler(GoogleCloudPilotAPI):
             method: int = DEFAULT_METHOD,
             headers: Dict[str, str] = None,
             project_id: str = None,
-            location: str = None,
-    ):
-        job_name = self._job_path(job=name, project_id=project_id, location=location)
+            use_oidc_auth: bool = True,
+            timeout_in_seconds: int = MAX_TIMEOUT,
+    ) -> scheduler.Job:
+        job_name = self._job_path(job=name, project_id=project_id)
         job = scheduler.Job(
             name=job_name,
             schedule=cron,
             time_zone=timezone or self.timezone,
+            attempt_deadline=timeout_in_seconds,
             http_target=scheduler.HttpTarget(
                 uri=url,
                 http_method=method,
                 body=payload.encode(),
                 headers=headers or {},
+                **(self.oidc_token if use_oidc_auth else {}),
             )
         )
 
@@ -89,15 +96,15 @@ class CloudScheduler(GoogleCloudPilotAPI):
         )
         return response
 
-    async def get(self, name: str, project_id: str = None, location: str = None):
-        job_name = self._job_path(job=name, project_id=project_id, location=location)
+    def get(self, name: str, project_id: str = None) -> scheduler.Job:
+        job_name = self._job_path(job=name, project_id=project_id)
         response = self.client.get_job(
             name=job_name,
         )
         return response
 
-    async def delete(self, name: str, project_id: str = None, location: str = None):
-        job_name = self._job_path(job=name, project_id=project_id, location=location)
+    async def delete(self, name: str, project_id: str = None) -> None:
+        job_name = self._job_path(job=name, project_id=project_id)
         response = self.client.delete_job(
             name=job_name,
         )
@@ -107,14 +114,15 @@ class CloudScheduler(GoogleCloudPilotAPI):
             self,
             name: str,
             url: str,
-            payload,
+            payload: str,
             cron: str,
             timezone: str = None,
             method: int = DEFAULT_METHOD,
             headers: Dict[str, str] = None,
             project_id: str = None,
-            location: str = None,
-    ):
+            use_oidc_auth: bool = True,
+            timeout_in_seconds: int = MAX_TIMEOUT,
+    ) -> scheduler.Job:
         try:
             response = await self.update(
                 name=name,
@@ -124,8 +132,9 @@ class CloudScheduler(GoogleCloudPilotAPI):
                 timezone=timezone,
                 method=method,
                 headers=headers,
-                location=location,
                 project_id=project_id,
+                use_oidc_auth=use_oidc_auth,
+                timeout_in_seconds=timeout_in_seconds,
             )
         except NotFound:
             response = await self.create(
@@ -136,7 +145,8 @@ class CloudScheduler(GoogleCloudPilotAPI):
                 timezone=timezone,
                 method=method,
                 headers=headers,
-                location=location,
                 project_id=project_id,
+                use_oidc_auth=use_oidc_auth,
+                timeout_in_seconds=timeout_in_seconds,
             )
         return response
